@@ -495,7 +495,7 @@ ioAction = liftIO
 setErrHandler :: (String -> IO ()) -> BrowserAction t ()
 setErrHandler h = modify (\b -> b { bsErr=h })
 
--- | @setErrHandler@ sets the IO action to call when
+-- | @setOutHandler@ sets the IO action to call when
 -- the browser chatters info on its running. To disable any
 -- such, set it to @const (return ())@.
 setOutHandler :: (String -> IO ()) -> BrowserAction t ()
@@ -809,7 +809,9 @@ request' nullVal rqState rq = do
    case e_rsp of
     Left v 
      | (reqRetries rqState < fromMaybe defaultMaxErrorRetries mbMx) && 
-       (v == ErrorReset || v == ErrorClosed) ->
+       (v == ErrorReset || v == ErrorClosed) -> do
+       --empty connnection pool in case connection has become invalid
+       modify (\b -> b { bsConnectionPool=[] })       
        request' nullVal rqState{reqRetries=succ (reqRetries rqState)} rq
      | otherwise -> 
        return (Left v)
@@ -911,7 +913,7 @@ request' nullVal rqState rq = do
 	         		   }
                              rq2
                  where
-                   newURI_abs = maybe newURI id (newURI `relativeTo` uri)
+                   newURI_abs = uriDefaultTo newURI uri
 
       (3,0,5) ->
         case retrieveHeaders HdrLocation rsp of
@@ -1033,7 +1035,11 @@ supportedScheme u = uriScheme u == "http:"
 -- If the second argument is not sufficient context for determining
 -- a full URI then anarchy reins.
 uriDefaultTo :: URI -> URI -> URI
+#if MIN_VERSION_network(2,4,0)
+uriDefaultTo a b = a `relativeTo` b
+#else
 uriDefaultTo a b = maybe a id (a `relativeTo` b)
+#endif
 
 
 -- This form junk is completely untested...
